@@ -14,8 +14,4 @@ typedef struct PSTRING {
   char* endnext;
 } PSTRING;
 
-PSTRING lowercase_pstring (PSTRING);
-void lowercase_pstring_inplace (PSTRING s);
-int is_pstring_true (PSTRING s);
-
 #endif /* pstring.h */
